using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class DragDestroyController : MonoBehaviour
{

    public static DragDestroyController instance { get; private set; }

    [SerializeField] Color disabledColor;
    [SerializeField] LayerMask fighterRaycastLayer;
    [SerializeField] float fighterRaycastLength = 20f;

    bool draggingEnabled = true;
    GameObject selectedEnemy;
    SpriteRenderer selectedEnemySprite;
    Vector3 originalEnemyPosition;
    int originalEnemyLayer;

    int noCollisionLayer;

    public void SetDraggingEnabled(bool enabled)
    {
        draggingEnabled = enabled;
    }

    void Update()
    {
        if (!RoundController.instance.roundRunning && draggingEnabled)
        {
            if (Input.GetButtonUp("Fire2") && selectedEnemy == null)
            {
                DeleteHoveredEnemy();
            }
            if (Input.GetButtonDown("Fire1"))
            {
                SelectHoveredEnemy();
            }
            if (Input.GetButtonUp("Fire1"))
            {
                PlaceSelectedEnemy();
            }
            MoveSelectedEnemy();
        }
    }

    GameObject RayCastForFighter(bool includeAntiHero)
    {
        Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
        RaycastHit2D hit = Physics2D.GetRayIntersection(ray, fighterRaycastLength, fighterRaycastLayer);

        if (hit.collider == null || (!includeAntiHero && hit.collider.gameObject.CompareTag("AntiPlayer")))
        {
            return null;
        }

        return hit.collider.transform.gameObject;
    }

    void DeleteHoveredEnemy()
    {
        GameObject enemy = RayCastForFighter(false);

        if (enemy != null)
        {
            StatsManager.instance.ModifyMana((int)enemy.GetComponent<Enemy>().opponentType.enemyCost);
            Destroy(enemy);
        }
    }

    void SelectHoveredEnemy()
    {
        GameObject enemy = RayCastForFighter(false);
        if (enemy != null)
        {
            selectedEnemy = enemy;
            originalEnemyPosition = enemy.transform.position;
            selectedEnemySprite = enemy.GetComponent<SpriteRenderer>();
            originalEnemyLayer = enemy.layer;
            // So that raycasts don't hit the current Enemy
            enemy.layer = noCollisionLayer;
        }

    }

    void MoveSelectedEnemy()
    {
        if (selectedEnemy != null)
        {
            // Move Enemy to mouse position
            selectedEnemy.transform.position = GetMouseWorldPosition() - new Vector3(0, selectedEnemySprite.bounds.extents.y);

            // Indicate that Enemy can't be placed on top of other Enemies
            selectedEnemySprite.color = RayCastForFighter(true) != null ? disabledColor : Color.white;
        }
    }

    void PlaceSelectedEnemy()
    {
        if (selectedEnemy != null)
        {
            if (RayCastForFighter(true) != null)
            {
                // When trying to place the Enemy on another Fighter, reset it to its original position
                selectedEnemy.transform.position = originalEnemyPosition;
                selectedEnemySprite.color = Color.white;
            }
            selectedEnemy.layer = originalEnemyLayer;
            selectedEnemy = null;
        }
    }

    Vector3 GetMouseWorldPosition()
    {
        Vector3 pos = Camera.main.ScreenToWorldPoint(Input.mousePosition);
        pos.z = selectedEnemy.transform.position.z;
        return pos;
    }

    void Awake()
    {
        if (instance == null)
        {
            instance = this;
        }
        else if (instance != this)
        {
            Destroy(gameObject);
        }
        DontDestroyOnLoad(gameObject);

        noCollisionLayer = LayerMask.NameToLayer("No Collision");
    }
}
