using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;



public class UpgradeController : MonoBehaviour
{
    [SerializeField] Button manaButton;

    [Header("Price Texts")]
    [SerializeField] Text healthText, damageText, armorText, manaText;

    [Header("Multiplier Texts")]
    [SerializeField] Text healthMultiplierText, damageMultiplierText, armorMultiplierText, manaMultiplierText;

    Dictionary<UpgradeTypes,Text> upgradeTexts = new Dictionary<UpgradeTypes,Text>();
    Dictionary<UpgradeTypes, Text> multiplierTexts = new Dictionary<UpgradeTypes, Text>();
    StatsManager statsManager;

    
    private void Awake()
    {
        statsManager = FindObjectOfType<StatsManager>();
        
        //initalise upgrade Texts
        upgradeTexts.Add(UpgradeTypes.ENEMY_HEALTH, healthText);
        upgradeTexts.Add(UpgradeTypes.ENEMY_DAMAGE, damageText);
        upgradeTexts.Add(UpgradeTypes.ENEMY_ARMOR, armorText);
        upgradeTexts.Add(UpgradeTypes.MANA_COST, manaText);
        UpdateAllCosts();

        //initalize Multiplier
        multiplierTexts.Add(UpgradeTypes.ENEMY_HEALTH,healthMultiplierText);
        multiplierTexts.Add(UpgradeTypes.ENEMY_DAMAGE, damageMultiplierText);
        multiplierTexts.Add(UpgradeTypes.ENEMY_ARMOR, armorMultiplierText);
        multiplierTexts.Add(UpgradeTypes.MANA_COST, manaMultiplierText);
        foreach (KeyValuePair<UpgradeTypes,Text> multiplierText in multiplierTexts)
        {
            multiplierText.Value.text = statsManager.GetTotalEffectMultiplier(multiplierText.Key).ToString("F2");
        }

    }
    public void Buy(string upgradeType)
    {
        switch (upgradeType)
        {
            case "Health":
                UpdateCostText(UpgradeTypes.ENEMY_HEALTH);
                break;
            case "Damage":
                UpdateCostText(UpgradeTypes.ENEMY_DAMAGE);
                break;
            case "Armor":
                UpdateCostText(UpgradeTypes.ENEMY_ARMOR);
                break;
            case "Mana":
                UpdateCostText(UpgradeTypes.MANA_COST);
                break;
        }
    }

    private void UpdateCostText(UpgradeTypes upgradeType)
    {
        if (statsManager.BuyUpgrade(upgradeType))
        {
            upgradeTexts[upgradeType].text = statsManager.GetNextUpgradeCost(upgradeType).ToString();
            multiplierTexts[upgradeType].text = statsManager.GetTotalEffectMultiplier(upgradeType).ToString("F2");
            if (upgradeType == UpgradeTypes.MANA_COST)
            {
                UpdateAllCosts();
                DeactivateMana();
            }
        }

    }

    private void UpdateAllCosts()
    {
        foreach (KeyValuePair<UpgradeTypes, Text> upgradeText in upgradeTexts)
        {
            upgradeText.Value.text = statsManager.GetNextUpgradeCost(upgradeText.Key).ToString();
        }
    }

    private void DeactivateMana()
    {
        if (statsManager.GetTotalEffectMultiplier(UpgradeTypes.MANA_COST) <= 0.1)
        {
            manaButton.interactable = false;
            multiplierTexts[UpgradeTypes.MANA_COST].text = "0.1";
            upgradeTexts[UpgradeTypes.MANA_COST].text = "";
        }
    }
}
